@@ -113,13 +113,10 @@ typedef enum {
 	GOK_EXPAND_ALWAYS
 } GokKeyboardExpandPolicy;
 
-/* maximum length of a keyboard name */
-#define MAX_KEYBOARD_NAME 50
-
 /* GokKeyboard structure */
 /* If you add data members to this structure, initialize them in gok_keyboard_new */
 typedef struct GokKeyboard {
-	gchar Name[MAX_KEYBOARD_NAME + 1];
+	gchar* Name;
 	KeyboardShape shape;
 	KeyboardLayouts LayoutType;
 	KeyboardTypes Type;
