@@ -101,7 +101,10 @@ gboolean gok_page_actions_initialize (GladeXML* xml)
 			/* TODO: persist this list so we can query it later */
 			slist = slist->next;
 		}
-	
+		if (!gok_input_get_extension_device_name() ||
+		    !strlen(gok_input_get_extension_device_name())) {
+		         items = g_list_prepend (items, "");
+		}
 		gtk_combo_set_popdown_strings (GTK_COMBO (pComboBox), items);
 		g_list_free (items);
 	}
@@ -552,7 +555,7 @@ void gok_page_actions_button_clicked_delete ()
 			GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_MESSAGE_QUESTION,
 			GTK_BUTTONS_YES_NO,
-			buffer);
+			"%s", buffer);
 	
 	gtk_window_set_title (GTK_WINDOW (pDialog), _("GOK Delete Action"));
 	result = gtk_dialog_run (GTK_DIALOG (pDialog));
@@ -718,7 +721,7 @@ void gok_page_actions_input_device_changed (GtkEditable* pEditControl)
 	if (pStrComboInputDeviceName && strlen (pStrComboInputDeviceName) == 0) {
 		g_free (pStrComboInputDeviceName);
 	}
-	else
+	else if (pStrComboInputDeviceName)
 	{
 		/* find the device in our list of input devices */
 	        
