@@ -160,6 +160,8 @@ static gchar ArrayCallNames[MAX_CALL_NAMES][24] ={
 "GetRate"
 };
 
+void gok_scanner_input_motion_xevie (gint x, gint y);
+
 static xmlChar *gok_lang = NULL;
 
 /**
@@ -1764,7 +1766,7 @@ gboolean gok_scanner_change_method (gchar* NameAccessMethod)
 	pAccessMethod = m_pAccessMethodFirst;
 	while (pAccessMethod != NULL)
 	{
-		if (g_strcasecmp (pAccessMethod->Name, NameAccessMethod) == 0)
+		if (gok_main_utf8_casecmp (pAccessMethod->Name, NameAccessMethod) == 0)
 		{
 			/* found the access method */
 			m_pAccessMethodCurrent = pAccessMethod;
@@ -2193,7 +2195,6 @@ void gok_scanner_on_key_enter (GokKey* pKey)
 
 	gok_main_raise_window ();
 	gok_scanner_timer_start_dwell();
-		
 	gok_scanner_perform_effects (m_pEffectsOnKeyEnter);
 }
 
@@ -2319,8 +2320,10 @@ void gok_scanner_mouse_movement (gint x, gint y)
 {
 	m_MouseX = x;
 	m_MouseY = y;
-
 	gok_scanner_perform_effects (m_pEffectsMouseMovement);
+	if (gok_main_get_use_xevie()) {
+		gok_scanner_input_motion_xevie (x, y);
+	}
 }
 
 /**
@@ -2370,10 +2373,106 @@ void gok_scanner_input_motion (gint *motion_data, gint n_axes)
 		}
 		m_pContainingKey = pKey;
 	}
-	gok_keyboard_paint_pointer (pKeyboard, 
-				    pWindow, x, y);
+	gok_keyboard_paint_pointer (pKeyboard, pWindow, x, y);
 }
 
+/**
+* gok_scanner_input_motion_libusb
+* @x: Horizontal (first axis) input device motion component.
+* @y: Vertical (second axis)  input device motion component.
+*
+* Handler for input device motion events generated by an input device accessed through libusb. 
+**/
+void gok_scanner_input_motion_libusb (gint x, gint y)
+{
+	GokKey *pKey;
+	gint w, h;
+	gint slop = gok_scanner_get_slop();
+	static gint offset_x = 0;
+	static gint offset_y = 0;
+	GokKeyboard *pKeyboard = gok_main_get_current_keyboard ();
+	GtkWidget *pWindow = gok_main_get_main_window ();
+	
+	gdouble multiplier = gok_data_get_valuator_sensitivity ();
+	x = x * multiplier;
+	y = y * multiplier;
+
+	/* clip coordinates to window */
+	gdk_drawable_get_size (pWindow->window, &w, &h);
+	/* horizontal */
+	if (w + slop < offset_x + x) {
+	        /* too far right */
+		x = w + slop - offset_x;
+	} else if (offset_x + x < -slop) {
+   	        /* too far left */
+		x = - offset_x - slop;
+	}
+	/* vertical */
+	if (h + slop < offset_y + y) {
+	        /* too far below */
+		y = h + slop - offset_y;
+	} else if (offset_y + y < -slop) {
+	        /* too far above */
+		y = -offset_y - slop;
+	}
+
+	offset_x += x;
+	offset_y += y;
+
+   	pKey = gok_keyboard_find_key_at (pKeyboard, offset_x, offset_y, m_pContainingKey);
+   	gok_keyboard_unpaint_pointer (pKeyboard, pWindow);
+	if (m_pContainingKey != pKey) {
+		gok_scanner_on_key_leave (m_pContainingKey);
+		if (pKey) {
+			gok_scanner_on_key_enter (pKey);
+		}
+		else if (m_pContainingKey) {
+		        gok_scanner_on_key_leave (m_pContainingKey);
+		}
+		m_pContainingKey = pKey;
+	}
+	gok_keyboard_paint_pointer (pKeyboard, pWindow, offset_x, offset_y);
+}
+
+/**
+* gok_scanner_input_motion_xevie
+* @x: Horizontal (first axis) input device motion component.
+* @y: Vertical (second axis)  input device motion component.
+*
+* Handler for input device motion events generated by an input device accessed through libusb. 
+**/
+void gok_scanner_input_motion_xevie (gint x, gint y)
+{
+	GokKey *pKey;
+	gint w, h;
+	static gint offset_x = 0, origin_x = 0;
+	static gint offset_y = 0, origin_y = 0;
+	GokKeyboard *pKeyboard = gok_main_get_current_keyboard ();
+	GtkWidget *pWindow = gok_main_get_main_window ();
+	
+	/* ignore motion events outside window */
+	gdk_drawable_get_size (pWindow->window, &w, &h);
+	gdk_window_get_position (pWindow->window, &offset_x, &offset_y);
+	gdk_window_get_root_origin (pWindow->window, &origin_x, &origin_y);
+	if (x < offset_x || x > offset_x+w ||
+	    y < offset_y || y > offset_y+h) {
+	  gok_scanner_on_key_leave (m_pContainingKey);
+	  return;
+	}
+   	pKey = gok_keyboard_find_key_at (pKeyboard, x-offset_x, y-offset_y, m_pContainingKey);
+	if (m_pContainingKey != pKey) {
+		gok_scanner_on_key_leave (m_pContainingKey);
+		if (pKey) {
+			gok_scanner_on_key_enter (pKey);
+		}
+		else if (m_pContainingKey) {
+		        gok_scanner_on_key_leave (m_pContainingKey);
+		}
+		m_pContainingKey = pKey;
+	}
+}
+
+
 /**
 * gok_scanner_on_timer1
 * @data: Pointer to the user data associated with the timer.
@@ -3340,7 +3439,6 @@ gboolean
 gok_scanner_current_state_uses_corepointer (void)
 {
 	gboolean retval = FALSE;
-	GokAccessMethod *method;
 
 	if  (m_pEffectsLeftButtonDown ||
 	     m_pEffectsLeftButtonUp ||
@@ -3362,7 +3460,21 @@ gok_scanner_current_state_uses_corepointer (void)
 				      GOK_GCONF_INPUT_DEVICE, 
 				      &input_device_name);
 
-		if (input_device_name == NULL)
+		/* IGNORE potential corepointer effects if we're using
+		 * direct selection and there is no specified XInput device;
+		 * since arguably it's worse to detach the corepointer
+		 * that just to allow the resulting bugs.
+		 * We make this determination because this is the default
+		 * config and thus the initial out-of-the-box experience
+		 * for GOK, and we don't want to alarm users, testers,
+		 * or distros by spewing dire warnings.  Besides, most
+		 * direct selection users will be able to work around
+		 * the corepointer-grab bugs, and the warnings
+		 * are still present in Help and the GOK docs, for those
+		 * few direct-selection users who may experience trouble
+		 * as a result of our suppressing the corepointer-detach.
+		 */
+		if ((input_device_name == NULL) && (strcmp (gok_data_get_name_accessmethod (), "directselection")))
 			retval = TRUE;
 		else
 			g_free (input_device_name);
@@ -3386,6 +3498,18 @@ gok_scanner_current_state_uses_corepointer (void)
 gboolean 
 gok_scanner_current_state_uses_core_mouse_button(int button)
 {
+	/* Special case dwell mode to always return FALSE?
+	 * This might improve interaction with 'clicker-like' tools
+	 * such as MouseTweaks. Waiting for related investigation to conclude.
+	 * see bug 538004 (and possibly 537827). Probably best if these tools
+	 * use gok in direcselection mode, which will probably require 538004 to
+	 * be fixed.
+	
+	if (!strcmp(m_pAccessMethodCurrent->Name,"dwellselection")){
+		return FALSE;
+	}	
+	 */
+	 
 	switch (button) {
 		case 1:
 			if (   (m_pEffectsLeftButtonDown != NULL)
@@ -3549,3 +3673,9 @@ gok_scanner_drop_refs (GokKey *pKey)
   if (m_pKeyEntered == pKey) m_pKeyEntered = NULL;
   if (m_pContainingKey == pKey) m_pContainingKey = NULL;
 }
+
+gboolean 
+gok_scanner_key_is_active (void)
+{
+  return m_pKeyEntered != NULL;
+}
