@@ -185,8 +185,6 @@ gok_spy_get_focussed_object (void)
  */
 void gok_spy_accessible_ref(Accessible* accessible)
 {
-	char *s;
-
 	gok_log_enter ();
 
 	if (accessible != NULL)
@@ -194,6 +192,7 @@ void gok_spy_accessible_ref(Accessible* accessible)
 		explicitrefs++;
 		Accessible_ref (accessible);
 #if defined(ENABLE_LOGGING_NORMAL)
+		char *s;
 		s = Accessible_getName (accessible);
 		if (s == NULL)
 		{
@@ -216,14 +215,13 @@ void gok_spy_accessible_ref(Accessible* accessible)
 
 void gok_spy_accessible_implicit_ref(Accessible* accessible)
 {
-	char *s;
-
 	gok_log_enter ();
 
 	if (accessible != NULL)
 	{
 		implicitrefs++;
 #if defined(ENABLE_LOGGING_NORMAL)
+		char *s;
 		s = Accessible_getName (accessible);
 		if (s == NULL)
 		{
@@ -246,13 +244,12 @@ void gok_spy_accessible_implicit_ref(Accessible* accessible)
 
 void gok_spy_accessible_unref(Accessible* accessible)
 {
-	char *s;
-
 	gok_log_enter ();
 
 	if (accessible != NULL)
 	{
 #if defined(ENABLE_LOGGING_NORMAL)
+		char *s;
 		s = Accessible_getName (accessible);
 		if (s == NULL)
 		{
@@ -1119,7 +1116,6 @@ gboolean _gok_spy_handle_gecko_browse_tab_change (Accessible* source)
 gboolean gok_spy_check_queues(void)
 {
 	EventNode* en = NULL;
-	char *s;
 	GokKeyboard *current_kbd;
 
 	gok_log_enter();
@@ -1137,6 +1133,7 @@ gboolean gok_spy_check_queues(void)
 #if defined(ENABLE_LOGGING_NORMAL)
 		if (en && en->event && en->event->source != NULL)
 		{
+			char *s;
 			s = Accessible_getName (en->event->source);
 			if (s == NULL)
 			{
@@ -1569,8 +1566,6 @@ GSList* gok_spy_append_node (GSList* nodes,
 			     Accessible* pAccessible,
 			     AccessibleNodeFlags flags)
 {
-	AccessibleNode* pLastNode = NULL;
-	AccessibleNode* pNewNode = NULL;
 	AccessibleRelation** relations = NULL;
 	Accessible* targetAccessible = NULL;
 	AccessibleRole role;
@@ -1963,7 +1958,7 @@ GSList* gok_spy_append_node (GSList* nodes,
 					    firstcellname = NULL;
 					}
 					/* use row index */
-					pName = g_strdup_printf ("%d",irow);
+					pName = g_strdup_printf ("%d", (gint) irow);
 				    }
 				}
 				if (rowstring) SPI_freeString (rowstring);
@@ -1973,19 +1968,18 @@ GSList* gok_spy_append_node (GSList* nodes,
 					Accessible_unref (col);
 				}
 				if (colstring) {
-				    gchar* free_string = pName;
 				    if (strlen (colstring)) {
+					gchar *free_string = pName;
 					/* use header name */
 					pName = g_strconcat (pName, " ", colstring, NULL);
 					g_free (free_string);
 				    }
-				    else
-					SPI_freeString (colstring);
+				    SPI_freeString (colstring);
 				}
 				else {
 				    gchar *free_string = pName;
 				    /* use col index */
-				    pName = g_strconcat (pName, " ", g_strdup_printf ("%d",icol), NULL);
+				    pName = g_strdup_printf ("%s %d", pName, (gint) icol);
 				    g_free (free_string);
 				}
 				AccessibleTable_unref (table);
@@ -2308,7 +2302,8 @@ void gok_spy_process_focus (Accessible* accessible)
 	role = Accessible_getRole (accessible);
 
 	if (accessible && (m_ptheWindowAccessible == NULL) && 
-	    gok_spy_is_menu_role (role))
+	    gok_spy_is_menu_role (role) &&
+	    !gok_main_get_disable_autobranch ())
 	{
 	    AccessibleNode *node = g_new0 (AccessibleNode, 1);
 	    gchar *name;
@@ -2352,7 +2347,6 @@ gok_spy_button_is_switch_trigger (int button)
 		return FALSE;
 	}
 	else {
-	        gboolean retval;
 		gok_log("gok_data_get_drive_corepointer() is FALSE");
 		return gok_scanner_current_state_uses_core_mouse_button(button);
 	}
@@ -2420,8 +2414,6 @@ gulong gok_spy_get_modmask (void)
  */
 void gok_spy_modifier_listener(const AccessibleEvent *event, void *user_data)
 {
-	const struct gok_modifier_keymask *m;
-       
 	keyboardmods = event->detail2;
 
 	gok_log_enter();
@@ -2514,7 +2506,7 @@ gok_spy_get_active_frame( )
 static gboolean
 gok_spy_find_and_append_toolbar_items (Accessible *root, AccessibleNodeFlags flags)
 {
-	Accessible *child, *tmp;
+	Accessible *child;
 	gint i, nchildren;
 	gboolean retval = FALSE;
 
