@@ -157,6 +157,7 @@ typedef struct _GokArgs {
 	char* usb_vid_pid;
 	gdouble valuator_sensitivity;
 	gboolean non_system_mouse;
+	gboolean disable_autobranch;
 } GokArgs;
 
 static GokArgs gok_args;
@@ -348,6 +349,15 @@ static const GOptionEntry options[] = {
 		N_("Attempt to use GOK without the system mouse"),
 		NULL
 	},
+	{
+		"disable-autobranch",
+		0,
+		0,
+		G_OPTION_ARG_NONE,
+		&gok_args.disable_autobranch,
+		N_("Disable automatic keyboard branching"),
+		NULL
+	},
 	/* End the list */
 	{	NULL}
 };
@@ -432,6 +442,7 @@ gok_args_init (GokArgs *args)
 	args->usb_vid_pid = NULL;
 	args->valuator_sensitivity = 0;
 	args->non_system_mouse = 0;
+	args->disable_autobranch = FALSE;
 }
 
 /* callback for gok window state events */
@@ -553,21 +564,48 @@ gint main (gint argc, gchar *argv[])
 	return 0;
 }
 
+/**
+ * gok_main_utf8_casecmp:
+ * @a: a UTF-8 encoded string
+ * @b: a UTF-8 encoded string
+ *
+ * Compares two strings for ordering using the linguistically correct
+ * rules for the current locale. Case-insensitve version.
+ *
+ * Returns: < 0 if a compares before b, 0 if they compare equal, > 0 if a compares after b.
+ **/
+gint
+gok_main_utf8_casecmp (const gchar *a, const gchar *b)
+{
+	gchar *a_fold, *b_fold;
+	gint res;
+
+	a_fold = g_utf8_casefold (a, -1);
+	b_fold = g_utf8_casefold (b, -1);
+
+	res = g_utf8_collate (a_fold, b_fold);
+
+	g_free (a_fold);
+	g_free (b_fold);
+
+	return res;
+}
+
 /* private helper */
 gint
 comparebasenames (gconstpointer a, gconstpointer b)
 {
-	gchar* a_file;
-	gchar* b_file;
+	gchar *a_file, *b_file;
 	gint returncode;
-	a_file = g_path_get_basename(a);
-	b_file = g_path_get_basename(b);
 
-	returncode = g_strcasecmp(a_file, b_file);
-	
+	a_file = g_path_get_basename (a);
+	b_file = g_path_get_basename (b);
+
+	returncode = gok_main_utf8_casecmp (a_file, b_file);
+
 	g_free (a_file);
 	g_free (b_file);
-	
+
 	return returncode;
 }
 
@@ -1098,15 +1136,19 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 {
 	GokKeyboard* pKeyboard = NULL;
 	Accessible *acc = NULL;
+	gboolean branch;
 
 	gok_log_enter();	
 
 	/* note: ref counting happens in gok-spy's listeners */
 	m_pForegroundWindowAccessible = pAccessible;
 
+	branch = !gok_main_get_disable_autobranch ();
+
 	/* reset the context menu until we receive a focus event */
 	/*gok_spy_set_context_menu_accessible (NULL, 0);*/
-	if (pAccessible && (Accessible_getRole (pAccessible) == SPI_ROLE_ALERT))
+	if (pAccessible && (Accessible_getRole (pAccessible) == SPI_ROLE_ALERT)
+	    && branch)
 	{
 	    AccessibleNode *node = g_malloc (sizeof (AccessibleNode));
 	    GokSpyUIFlags ui_flags;
@@ -1123,7 +1165,8 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 	}
 	else if (pAccessible && Accessible_getChildCount (pAccessible) == 1
 	    && (acc = Accessible_getChildAtIndex (pAccessible, 0))
-	    && Accessible_getRole (acc) == SPI_ROLE_MENU) 
+	    && Accessible_getRole (acc) == SPI_ROLE_MENU
+	    && branch)
 	{
 	    AccessibleNode *node = g_malloc (sizeof (AccessibleNode));
 	    GokSpyUIFlags menu_flags;
@@ -1557,7 +1600,11 @@ gboolean gok_main_display_scan_previous_premade ()
 	if (pKeyboard->bDynamicallyCreated != TRUE)
 	{
 		gok_log_leave();
-		return gok_main_display_scan_previous();
+
+		if (gok_main_get_disable_autobranch ())
+			return FALSE;
+		else
+			return gok_main_display_scan_previous ();
 	}
 
 	/* initialize this array */
@@ -1689,7 +1736,7 @@ gok_main_keyboard_find_byname (const gchar *NameKeyboard)
 		pKB = m_pKeyboardFirst;
 		while (pKB != NULL)
 		{
-			if (g_strcasecmp (pKB->Name, NameKeyboard) == 0)
+			if (gok_main_utf8_casecmp (pKB->Name, NameKeyboard) == 0)
 			{
 				/* found the keyboard */
 				break;
@@ -2982,6 +3029,20 @@ gok_main_get_login (void)
 	return gok_args.is_login;
 }
 
+/**
+ * gok_main_get_disable_autobranch:
+ *
+ * Call this to see if the disable-autobranch argument was passed
+ * in from the command line.
+ *
+ * Returns: TRUE if auto-branching is disabled.
+ **/
+gboolean
+gok_main_get_disable_autobranch (void)
+{
+	return gok_args.disable_autobranch;
+}
+
 /**
 * gok_main_get_inputdevice_name
 * 
